@@ -24,7 +24,7 @@ pub const TRACE:usize = 0;
 pub struct Gsym // struct for a grammar symbol
 {
   pub sym : String,
-  pub rusttype : String,  //used only to indicate "mut"
+  pub rusttype : String, // used to derive private enum
   pub terminal : bool,
   pub label : String,  // object-level variable holding value
   pub precedence : i32,   // negatives indicate right associativity
@@ -38,7 +38,7 @@ impl Gsym
       sym : s.to_owned(),
       terminal : isterminal,
       label : String::default(),
-      rusttype : String::from("String"),
+      rusttype : String::new(),
       precedence : DEFAULTPRECEDENCE, // + means left, - means right
     }
   }
@@ -103,6 +103,9 @@ pub struct Grammar
   pub Errsym : String,        // error recovery terminal symbol
   pub Lexnames : HashMap<String,String>, // print names of grammar symbols
   pub Extras : String,        // indicated by {% .. %}, mostly  use ...
+  pub sametype: bool,  // determine if absyntype is only valuetype
+  pub lifetime: String,
+  pub tracelev:usize,
 }
 
 impl Grammar
@@ -125,6 +128,9 @@ impl Grammar
        Errsym : String::new(),
        Lexnames : HashMap::new(),
        Extras: String::new(),
+       sametype:true,
+       lifetime:String::new(), // empty means inferred
+       tracelev:1,
      }
   }//new grammar
 
@@ -197,6 +203,7 @@ impl Grammar
          let stokens:Vec<&str> = toksplit.collect();
          if stokens.len()<1 {continue;}
          match stokens[0] {
+         /*  deprecated by !
             "use" => {
               self.Extras.push_str("use ");
               self.Extras.push_str(stokens[1]);
@@ -207,12 +214,13 @@ impl Grammar
               self.Extras.push_str(stokens[2]);
               self.Extras.push_str("\n");              
             },
+         */            
             "!" => {
                let pbi = line.find('!').unwrap();
                self.Extras.push_str(&line[pbi+1..]);
                self.Extras.push_str("\n");                             
             },
-            "grammarname" | "grammar_name" => {
+            "grammarname" => {
                self.name = String::from(stokens[1]);
             },
             "EOF" => {atEOF=true},
@@ -232,24 +240,23 @@ impl Grammar
                   tokentype.push_str(&stokens[i][..]);
                   tokentype.push(' ');
                }
-	       if stokens.len()>2 {
-                 newterm.settype(tokentype.trim());
-               }
-               else {newterm.settype(&self.Absyntype);}
+               newterm.settype(tokentype.trim());
                self.Symhash.insert(stokens[1].to_owned(),self.Symbols.len());
                self.Symbols.push(newterm);           
 	    }, //typed terminals
-	    "nonterminal" if stage==0 => {
+	    "nonterminal" if stage==0 => {   // with type
 	       let mut newterm = Gsym::new(stokens[1],false);
                let mut tokentype = String::new();
                for i in 2..stokens.len() {
                   tokentype.push_str(&stokens[i][..]);
                   tokentype.push(' ');
                }
-	       if stokens.len()>2 {
-                 newterm.settype(tokentype.trim());
-               }
-               else {newterm.settype(&self.Absyntype);}
+//	       if stokens.len()>2 && stokens[2]!="mut" {
+//	         let rtype = format!("PE_Variant_{}({})",self.Symbols.len(),tokentype.trim());
+//		 newterm.settype(&rtype);
+               newterm.settype(tokentype.trim());
+//               }
+//               else {newterm.settype(&self.Absyntype);}
                self.Symhash.insert(stokens[1].to_owned(),self.Symbols.len());
                self.Symbols.push(newterm);
                self.Rulesfor.insert(stokens[1].to_owned(),HashSet::new());
@@ -305,6 +312,11 @@ impl Grammar
                   self.Resynch.insert(stokens[i].trim().to_owned());
                } // for each subsequent token
             },
+            "lifetime" if stokens.len()==2 && stokens[1].len()>0 => {
+               
+               self.lifetime = if &stokens[1][0..1]=="'" && stokens[1].len()>1 
+                 {String::from(stokens[1])} else {format!("'{}",stokens[1])};
+            },
             "absyntype" | "valuetype" if stage==0 => {
                let pos = line.find(stokens[0]).unwrap() + stokens[0].len();
                self.Absyntype = String::from(line[pos..].trim());
@@ -350,10 +362,6 @@ impl Grammar
 
               let pos0 = line.find(stokens[1]).unwrap() + stokens[1].len();
               let mut linec = line[pos0..].to_string();
-//              let semaction = if let Some(pos) = line.find('{') {
-//                 linec.truncate(pos-pos0);
-//                 &line[pos+1..]
-//              } else {"}"};
               let barsplit:Vec<_> = linec.split('|').collect();
               
               for rul in &barsplit 
@@ -374,7 +382,7 @@ impl Grammar
 		   break;
                 }
 		let toks:Vec<&str> = strtok.split(':').collect();
-if TRACE>2&&toks.len()>1 {println!("see labeled token {}",strtok);}		
+//if TRACE>2&&toks.len()>1 {println!("see labeled token {}",strtok);}		
 		match self.Symhash.get(toks[0]) {
 		   None => {panic!("unrecognized grammar symbol {}, line {}",toks[0],linenum); },
 		   Some(symi) => {
@@ -387,8 +395,34 @@ if TRACE>2&&toks.len()>1 {println!("see labeled token {}",strtok);}
                        panic!("Only terminal symbols may follow the error recovery symbol {}, line {}",&self.Errsym, linenum);
                      }
 		     let mut newsym = sym.clone();
-		     if toks.len()>1 && toks[1].trim().len()>0
-                        { newsym.setlabel(toks[1].trim()); }
+		     
+		     if toks.len()>1 && toks[1].trim().len()>0 { //label exists
+		       let mut label = String::new();
+		       
+		       if let Some(atindex) = toks[1].find('@') { //if-let pattern
+			 label.push_str(toks[1]);
+			 while !label.ends_with('@') && i<bstokens.len()
+			 { // i indexes all tokens split by whitespaces
+			    label.push(' '); label.push_str(bstokens[i]); i+=1;
+			 }
+			 if !label.ends_with('@') { panic!("pattern labels must be closed with @, line {}",linenum);}			 
+		       } // if-let pattern
+/*		       
+                       if &toks[1][0..1]=="'" { // if-let pattern
+		         label.push_str(&toks[1][..]); // include leading ' in string
+			 while !label.ends_with("'") && i<bstokens.len()
+			 { // i indexes all tokens split by whitespaces
+			    label.push(' '); label.push_str(bstokens[i]); i+=1;
+			 }
+			 if !label.ends_with("'") { panic!("pattern labels must be closed with  a ', line {}",linenum);}
+		       }// ' pattern '
+*/		       
+                       else { label = toks[1].trim().to_string(); }
+		       newsym.setlabel(label.trim_end_matches('@'));
+		       //newsym.setlabel(label.trim_end_matches("'"));		       
+		       //newsym.setlabel(toks[1].trim()); 
+	             }//label exists
+			
                      if maxprec.abs() < newsym.precedence.abs()  {
                         maxprec=newsym.precedence;
                      }
@@ -438,11 +472,20 @@ if TRACE>2&&toks.len()>1 {println!("see labeled token {}",strtok);}
         rhs:vec![topgsym.clone()], //,eofterm],  //eofterm is lookahead
         action: String::default(),
         precedence : 0,
-//        Ruleaction: |p|{AT::default()}, //{p.Parsestack.pop().unwrap().value},
      };
      self.Rules.push(startrule);  // last rule is start rule
-     if TRACE>0 {println!("{} rules in grammar",self.Rules.len());}
-     if self.Externtype.len()<1 {self.Externtype = self.Absyntype.clone();} ////***
+     if self.tracelev>0 {println!("{} rules in grammar",self.Rules.len());}
+     if self.Externtype.len()<1 {self.Externtype = self.Absyntype.clone();}
+     // compute sametype value (default true)
+     for ri in 0..self.Symbols.len()
+     {
+        if self.Symbols[ri].rusttype.len()<2 {
+          self.Symbols[ri].settype(&self.Absyntype);
+        }
+        else if &self.Symbols[ri].rusttype!=&self.Absyntype {
+          self.sametype = false;
+        }
+     }//compute sametype
   }//parse_grammar
 }// impl Grammar
 // last rule is always start rule and first state is start state
