@@ -403,7 +403,7 @@ impl Statemachine
      match currentaction {
         Some(Accept) => {
           changefsm = false;
-          if !isaccept {println!("Reduce({})-Accept conflict resolved in favor of Accept",ri1)}
+          //if !isaccept {println!("Reduce({})-Accept conflict resolved in favor of Accept",ri1)}
         },
         Some(Reduce(ri2)) if ri2<ri1 && !isaccept => {
            changefsm=false;
